import 'package:flutter/material.dart';

import 'bee_header.dart';
import 'bee_item.dart';
import 'bee_title.dart';
import 'model/bee_column.dart';

abstract class BeeItemCel {
  bool selection = false;

  dynamic get({required String propertyName}) {}
}

class BeeDataGrid extends StatefulWidget {
  final List<BeeColumn> columns;
  final List<BeeItemCel> rows;
  final String? title;
  final int rowsPerPage;

  const BeeDataGrid(
      {Key? key,
      required this.columns,
      required this.rows,
      this.title,
      this.rowsPerPage = 10})
      : super(key: key);

  @override
  _BeeDataGridState createState() => _BeeDataGridState();
}

class _BeeDataGridState extends State<BeeDataGrid> {
  final _verticalScrollReceiver = ScrollController();
  final _verticalScrollSender = ScrollController();
  final _horizontalScrollReceiver = ScrollController();
  final _horizontalScrollSender = ScrollController();
  late double _barOffset;
  late double _barHorizontalOffset;
  late double _viewOffset;
  late double _viewHorizontalOffset;
  static const _cellWidth = 80.0;
  static const _cellHeight = 40.0;
  static const _cellHeightHeader = 40.0;
  static const _cellColumWidth = 300.0;
  static const _axisWidth = 40.0;
  static const _cellColumYWidth = 100.0;
  List<bool> _isCurrentPage = [];
  int _page = 0;
  List<int> _range = [-1, -1];
  List<BeeItemCel> listSelection = [];
  double _editOpacity = 0.50;
  double _deleteOpacity = 0.50;
  bool _isOpacityEdit = true;
  bool _isOpacityDel = true;
  final _opacity = 0.50;
  final _notOpacity = 1.0;

  //VERTICAL BAR

  double get barMinScrollExtent => 0.0;

  double get barMaxScrollExtent =>
      context.size!.height - (_cellHeight + _axisWidth + 40);

  double get viewMaxScrollExtent =>
      _verticalScrollReceiver.position.maxScrollExtent;

  double get viewMinScrollExtent =>
      _verticalScrollReceiver.position.minScrollExtent;

  //HORIZONTAL BAR

  double get barHorizontalMinScrollExtent => 0.0;

  double get barHorizontalMaxScrollExtent =>
      context.size!.width - (_cellColumYWidth);

  double get viewHorizontalMaxScrollExtent =>
      _horizontalScrollReceiver.position.maxScrollExtent;

  double get viewHorizontalMinScrollExtent =>
      _horizontalScrollReceiver.position.minScrollExtent;

  @override
  void initState() {
    super.initState();

    //VERTICAL BAR
    _barOffset = 0.0;
    _viewOffset = 0.0;
    _verticalScrollReceiver.addListener(_updateAxisVerticalOffset);

    //HORIZONTAL BAR
    _barHorizontalOffset = 0.0;
    _viewHorizontalOffset = 0.0;
    _horizontalScrollReceiver.addListener(_updateAxisHorizontalOffset);

    //PAGINADOS
    var _rows = widget.rows.length;
    var _rowsPerPage = widget.rowsPerPage;

    //LLENADO DE LISTAS
    _isCurrentPage =
        List<bool>.filled((_rows / _rowsPerPage).ceil() + 2, false);
    _isCurrentPage[1] = true;
  }

  @override
  void dispose() {
    _verticalScrollReceiver.removeListener(_updateAxisVerticalOffset);
    _horizontalScrollReceiver.removeListener(_updateAxisHorizontalOffset);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    var _rows = widget.rows.length;
    var _rowsPerPage = widget.rowsPerPage;
    var _lastRow = _page * _rowsPerPage + _rowsPerPage;
    if (_lastRow > _rows) {
      _lastRow = _rows;
    }
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 20),
      child: Column(
        children: [
          widget.title != null
              ? BeeTitle("Itinerarios")
              : SizedBox(
                  height: 62,
                ),
          Row(
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              Visibility(
                visible: true,
                child: Padding(
                  padding: const EdgeInsets.only(right: 5, top: 5, bottom: 5),
                  child: OutlinedButton.icon(
                    onPressed: () => print("Agregar"),
                    icon: Icon(Icons.add, color: Colors.white),
                    label: Text(
                      'Agregar',
                      style: TextStyle(color: Colors.white),
                    ),
                    style:
                        OutlinedButton.styleFrom(backgroundColor: Colors.green),
                  ),
                ),
              ),
              Visibility(
                visible: true,
                child: Padding(
                  padding: const EdgeInsets.only(right: 5, top: 5, bottom: 5),
                  child: AnimatedOpacity(
                    duration: Duration(milliseconds: 350),
                    opacity: _editOpacity,
                    child: OutlinedButton.icon(
                      onPressed: () => print("Editar"),
                      icon: Icon(Icons.edit, color: Colors.white),
                      label: Text(
                        'Editar',
                        style: TextStyle(color: Colors.white),
                      ),
                      style: OutlinedButton.styleFrom(
                          backgroundColor: Colors.orangeAccent),
                    ),
                  ),
                ),
              ),
              Visibility(
                visible: true,
                child: Padding(
                  padding: const EdgeInsets.only(right: 5, top: 5, bottom: 5),
                  child: AnimatedOpacity(
                    duration: Duration(milliseconds: 350),
                    opacity: _deleteOpacity,
                    child: OutlinedButton.icon(
                      onPressed: () => print("Eliminar"),
                      icon: Icon(Icons.delete, color: Colors.white),
                      label: Text(
                        'Eliminar',
                        style: TextStyle(color: Colors.white),
                      ),
                      style: OutlinedButton.styleFrom(
                          backgroundColor: Colors.redAccent),
                    ),
                  ),
                ),
              ),
            ],
          ),
          Expanded(
            child: Column(
              children: <Widget>[
                SizedBox(
                  height: _cellHeightHeader,
                  child: Row(
                    children: <Widget>[
                      _buildCorner(),
                      Expanded(
                        child: _buildXLabels(),
                      ),
                    ],
                  ),
                ),
                SizedBox(
                  height: _cellHeight * widget.rowsPerPage + 10,
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      SizedBox(
                        child: _buildYLabels(_rowsPerPage, _lastRow),
                        width: _cellColumYWidth,
                      ),
                      Expanded(
                        child: _buildGrid(_rowsPerPage, _lastRow),
                      ),
                    ],
                  ),
                ),
                Padding(
                  padding: EdgeInsets.symmetric(vertical: 5),
                ),
                Row(
                  mainAxisAlignment: MainAxisAlignment.end,
                  children: [
                    InkWell(
                      onTap: _moveLeft,
                      child: Container(
                        decoration: BoxDecoration(
                            color: Colors.amber[300],
                            borderRadius: BorderRadius.only(
                              topLeft: Radius.circular(5),
                              bottomLeft: Radius.circular(5),
                            )),
                        height: 40,
                        width: 40,
                        child: Center(
                            child: Icon(
                          Icons.arrow_back,
                          color: Colors.white,
                          size: 20,
                        )),
                      ),
                    ),
                    VerticalDivider(),
                    InkWell(
                      onTap: _moveRigth,
                      child: Container(
                        decoration: BoxDecoration(
                            color: Colors.amber[300],
                            borderRadius: BorderRadius.only(
                              topRight: Radius.circular(5),
                              bottomRight: Radius.circular(5),
                            )),
                        height: 40,
                        width: 40,
                        child: Center(
                            child: Icon(
                          Icons.arrow_forward,
                          color: Colors.white,
                          size: 20,
                        )),
                      ),
                    ),
                  ],
                ),
              ],
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(top: 8, bottom: 8),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Text(
                  'Mostrando desde ${(_page * _rowsPerPage) + 1} hasta $_lastRow de ${_rows} registros',
                  style: TextStyle(fontWeight: FontWeight.bold),
                ),
                Material(
                  elevation: 4.0,
                  borderRadius: BorderRadius.all(Radius.circular(6.0)),
                  child: ToggleButtons(
                      constraints: BoxConstraints(minHeight: 32, minWidth: 32),
                      children: [
                        Icon(
                          Icons.arrow_left_outlined,
                        ),
                        ...List<Widget>.generate((_rows / _rowsPerPage).ceil(),
                            (int index) => Text('${(index + 1)}')),
                        Icon(
                          Icons.arrow_right_outlined,
                        )
                      ],
                      color: Colors.black,
                      selectedColor: Colors.blueAccent[300],
                      isSelected: _isCurrentPage,
                      onPressed: (int index) {
                        setState(() {
                          for (int buttonIndex = 0;
                              buttonIndex < _isCurrentPage.length;
                              buttonIndex++) {
                            if (buttonIndex == index) {
                              _isCurrentPage[buttonIndex] =
                                  !_isCurrentPage[buttonIndex];
                            } else {
                              _isCurrentPage[buttonIndex] = false;
                            }
                          }

                          var _pagination = (_rows / _rowsPerPage).ceil();

                          if (index == 0 || index == 1) {
                            _page = 0;
                          } else if (index == _pagination + 1) {
                            _page = _pagination - 1;
                          } else {
                            _page = index - 1;
                          }
                        });
                      }),
                )
              ],
            ),
          )
        ],
      ),
    );
  }

  Widget _buildCorner() {
    return Container(
      width: _cellColumYWidth,
      child: BeeHeader(data: "#"),
    );
  }

  Widget _buildXLabels() {
    return ListView.builder(
      scrollDirection: Axis.horizontal,
      controller: _horizontalScrollSender,
      physics: const NeverScrollableScrollPhysics(),
      itemCount: widget.columns.length,
      itemBuilder: (context, index) => Visibility(
        visible: widget.columns[index].show,
        child: SizedBox(
            width: widget.columns[index].width,
            child: BeeHeader(data: widget.columns[index].label!)),
      ),
    );
  }

  Widget _buildYLabels(int _rowsPerPage, int _lastRow) {
    return ListView.builder(
      scrollDirection: Axis.vertical,
      controller: _verticalScrollSender,
      physics: const NeverScrollableScrollPhysics(),
      itemCount: widget.rows.sublist(_page * _rowsPerPage, _lastRow).length,
      itemBuilder: (context, index) => SizedBox(
          width: _cellColumYWidth,
          child: InkWell(
            onTap: () => _toSelect(_rowsPerPage, _lastRow, index),
            child: BeeItem(
                selection: widget.rows
                    .sublist(_page * _rowsPerPage, _lastRow)[index]
                    .selection,
                data: "${(_page * _rowsPerPage) + index + 1}"),
          )),
    );
  }

  Widget _buildGrid(int _rowsPerPage, int _lastRow) {
    return SizedBox(
      height: _cellHeight * widget.rowsPerPage + 10,
      child: ListView.builder(
        scrollDirection: Axis.horizontal,
        controller: _horizontalScrollReceiver,
        physics: const AlwaysScrollableScrollPhysics(),
        itemCount: widget.columns.length,
        itemBuilder: (context, indexColumn) => Visibility(
          visible: widget.columns[indexColumn].show,
          child: SizedBox(
            width: widget.columns[indexColumn].width,
            child: ListView.builder(
              physics: const NeverScrollableScrollPhysics(),
              itemCount:
                  widget.rows.sublist(_page * _rowsPerPage, _lastRow).length,
              itemBuilder: (context, indexRow) => InkWell(
                onTap: () => _toSelect(_rowsPerPage, _lastRow, indexRow),
                child: BeeItem(
                    selection: widget.rows
                        .sublist(_page * _rowsPerPage, _lastRow)[indexRow]
                        .selection,
                    data: widget.rows
                        .sublist(_page * _rowsPerPage, _lastRow)[indexRow]
                        .get(propertyName: widget.columns[indexColumn].field)),
              ),
            ),
          ),
        ),
      ),
    );
  }

  double getScrollViewDelta(
    double barDelta,
    double barMaxScrollExtent,
    double viewMaxScrollExtent,
  ) {
    return barDelta * viewMaxScrollExtent / barMaxScrollExtent;
  }

  void _moveLeft() {
    _horizontalScrollReceiver.animateTo(
        _horizontalScrollReceiver.offset - _cellColumWidth,
        duration: Duration(milliseconds: 500),
        curve: Curves.linear);
  }

  void _moveRigth() {
    _horizontalScrollReceiver.animateTo(
        _horizontalScrollReceiver.offset + _cellColumWidth,
        duration: Duration(milliseconds: 500),
        curve: Curves.linear);
  }

  void _updateAxisHorizontalOffset() {
    _horizontalScrollSender.jumpTo(_horizontalScrollReceiver.offset);
  }

  //VERTICAL BAR
  void _updateAxisVerticalOffset() {
    _verticalScrollSender.jumpTo(_verticalScrollReceiver.offset);
  }

  void _toSelect(int _rowsPerPage, int _lastRow, int indexRow) {
    setState(() {
      widget.rows.sublist(_page * _rowsPerPage, _lastRow)[indexRow].selection =
          !widget.rows
              .sublist(_page * _rowsPerPage, _lastRow)[indexRow]
              .selection;

      if (_isOpacityEdit) {
        _editOpacity = _notOpacity;
        _isOpacityEdit = !_isOpacityEdit;
      } else {
        _editOpacity = _opacity;
        _isOpacityEdit = !_isOpacityEdit;
      }

      if (_isOpacityDel) {
        _deleteOpacity = _notOpacity;
        _isOpacityDel = !_isOpacityDel;
      } else {
        _deleteOpacity = _opacity;
        _isOpacityDel = !_isOpacityDel;
      }

      print("index ${(_page * _rowsPerPage) + indexRow}");

      int index = (_page * _rowsPerPage) + indexRow;

      if (_range[0] == -1) {
        _range[0] = (_page * _rowsPerPage) + indexRow;
      } else {
        if (_range[1] == -1) {
          _range[1] = (_page * _rowsPerPage) + indexRow;
        }
      }

      if (_range[0] != -1 && _range[1] != -1) {
        for (int i = _range[0]; i <= _range[1]; i++) {
          widget.rows.sublist(_page * _rowsPerPage, _lastRow)[i].selection =
              true;
        }
        _range[1] = -1;
      }

      print("range: $_range");
    });
  }
}
